//+------------------------------------------------------------------+
//|                                                     Laguerre.mq4 |
//|                                                     Emerald King |
//|                                     mailto:info@emerald-king.com |
//+------------------------------------------------------------------+
#property copyright "Emerald King"
#property link      "mailto:info@emerald-king.com"

#property indicator_separate_window
#property indicator_buffers    1
#property indicator_color1     clrMagenta
#property indicator_width1     3
#property indicator_level1     0.75
#property indicator_level2     0.45
#property indicator_level3     0.15
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor clrDarkGray
#property strict

//
//
//

enum enTimeFrames
{
         tf_cu  = 0,                                           // Current time frame
         tf_m1  = PERIOD_M1,                                   // 1 minute
         tf_m5  = PERIOD_M5,                                   // 5 minutes
         tf_m15 = PERIOD_M15,                                  // 15 minutes
         tf_m30 = PERIOD_M30,                                  // 30 minutes
         tf_h1  = PERIOD_H1,                                   // 1 hour
         tf_h4  = PERIOD_H4,                                   // 4 hours
         tf_d1  = PERIOD_D1,                                   // Daily
         tf_w1  = PERIOD_W1,                                   // Weekly
         tf_mn1 = PERIOD_MN1,                                  // Monthly
         tf_n1  = -1,                                          // First higher time frame
         tf_n2  = -2,                                          // Second higher time frame
         tf_n3  = -3,                                          // Third higher time frame
         tf_cus = 12345678                                     // Custom time frame
      };
input enTimeFrames        inpTimeFrame           = tf_cu; 
input double              gamma                  = 0.66;              // Laguerre period
input ENUM_APPLIED_PRICE  Price                  = PRICE_CLOSE;       // Price to use
input bool                ShowArrows             = true;              // Show arrows
input string              arrowsId               = "lag arrows1";     // Arrows ID
input bool                arrowsOnEveryBreak     = true;              // Arrows on every signal
input bool                arrowsOnNewest         = false;             // Arrows drawn on newest bar of higher time frame bar
input double              arrowsDisplaceUp       = 0.5;               // Arrow gap up
input double              arrowsDisplaceDn       = 0.5;               // Arrow gap down
input bool                arrowsOnZoneEnter      = true;              // Arrows on entering OB/OS zone true/false?
input color               arrowsUpZoneEnterColor = clrBlue;           // Arrows on entering OB/OS zone up color
input color               arrowsDnZoneEnterColor = clrRed;            // Arrows on entering OB/OS zone down
input int                 arrowsUpZoneEnterCode  = 159;
input int                 arrowsDnZoneEnterCode  = 159;
input int                 arrowsUpZoneEnterSize  = 2;                 // Arrows on entering OB/OS zone up size
input int                 arrowsDnZoneEnterSize  = 2;                 // Arrows on entering OB/OS zone down size
input bool                arrowsOnZoneExit       = true;              // Arrows on exiting OB/OS zone true/false?           
input color               arrowsUpZoneExitColor  = clrLimeGreen;      // Arrows on exiting OB/OS zone up color
input color               arrowsDnZoneExitColor  = clrOrange;         // Arrows on exiting OB/OS zone down color 
input int                 arrowsUpZoneExitCode   = 119;               // Arrows on exiting OB/OS zone up code
input int                 arrowsDnZoneExitCode   = 119;               // Arrows on exiting OB/OS zone down code
input int                 arrowsUpZoneExitSize   = 2;                 // Arrows on exiting OB/OS zone up size
input int                 arrowsDnZoneExitSize   = 2;                 // Arrows on exiting OB/OS zone down size
input bool                Interpolate            = true;     

double val[],trend[],cnt[];
struct sGlobalStruct
{
   string indicatorFileName;
   int    indicatorTimeFrame;
};
sGlobalStruct global;
#define _mtfCall(_buff,_ind) iCustom(NULL,global.indicatorTimeFrame,global.indicatorFileName,tf_cu,gamma,Price,ShowArrows,arrowsId,arrowsOnEveryBreak,arrowsOnNewest,arrowsDisplaceUp,arrowsDisplaceDn,arrowsOnZoneEnter,arrowsUpZoneEnterColor,arrowsDnZoneEnterColor,arrowsUpZoneEnterCode,arrowsDnZoneEnterCode,arrowsUpZoneEnterSize,arrowsDnZoneEnterSize,arrowsOnZoneExit,arrowsUpZoneExitColor,arrowsDnZoneExitColor,arrowsUpZoneExitCode,arrowsDnZoneExitCode,arrowsUpZoneExitSize,arrowsDnZoneExitSize,_buff,_ind)
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
      
int OnInit()
{
   IndicatorBuffers(3);
   SetIndexBuffer(0,val,INDICATOR_DATA);
   SetIndexBuffer(1,trend,INDICATOR_CALCULATIONS);
   SetIndexBuffer(2,cnt  ,INDICATOR_CALCULATIONS);
   
   global.indicatorFileName  = WindowExpertName();
   global.indicatorTimeFrame = (enTimeFrames)timeFrameValue(inpTimeFrame);
   
   IndicatorSetString(INDICATOR_SHORTNAME,timeFrameToString(global.indicatorTimeFrame)+" Laguerre ("+(string)gamma+")");
return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason)  {  ObjectsDeleteAll(0,arrowsId+":"); }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int  OnCalculate(const int rates_total,
                 const int prev_calculated,
                 const datetime &time[],
                 const double &open[],
                 const double &high[],
                 const double &low[],
                 const double &close[],
                 const long &tick_volume[],
                 const long &volume[],
                 const int &spread[])
{
   int limit = (prev_calculated>0) ? rates_total-prev_calculated : rates_total-1; cnt[0] = limit;   
      if (global.indicatorTimeFrame !=_Period)
      {
         limit = (int)fmax(limit,fmin(rates_total-1,_mtfCall(2,0)*global.indicatorTimeFrame/_Period));
         for (int i=limit;i>=0 && !_StopFlag; i--)
         {
             int y = iBarShift(NULL,global.indicatorTimeFrame,time[i]);
                val[i] = _mtfCall(0,y);
                
                //
                //
                //
                     
                if (!Interpolate || (i>0 && y==iBarShift(NULL,global.indicatorTimeFrame,time[i-1]))) continue;
                  #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
                  int n,k; datetime btime = iTime(NULL,global.indicatorTimeFrame,y);
                     for(n = 1; (i+n)<rates_total && time[i+n] >= btime; n++) continue;	
                     for(k = 1; k<n && (i+n)<rates_total && (i+k)<rates_total; k++) _interpolate(val);     
          }          
   return(rates_total);
   }
   
   //
   //
   //
   
   struct laStruct
   {
     double L0;
     double L1;
     double L2;
     double L3;
   };
   static laStruct wrk[];
   static int         wrkSize = -1;
                if (wrkSize<rates_total) wrkSize = ArrayResize(wrk,rates_total+500);
      
   //
   //
   //
   
   for(int i=limit, r=rates_total-i-1; i>=0; i--,r++)
   {
      double prc = iMA(_Symbol,_Period,1,0,MODE_SMA,Price,i);
	   double cu = 0.00,cd = 0.00;
	   if (r>0)
      {      
         wrk[r].L0 = (1.0 - gamma)*prc + gamma*wrk[r-1].L0;
	      wrk[r].L1 = -gamma*wrk[r].L0  + wrk[r-1].L0 + gamma*wrk[r-1].L1;
	      wrk[r].L2 = -gamma*wrk[r].L1  + wrk[r-1].L1 + gamma*wrk[r-1].L2;
	      wrk[r].L3 = -gamma*wrk[r].L2  + wrk[r-1].L2 + gamma*wrk[r-1].L3;
	      
	      if (wrk[r].L0 >= wrk[r].L1)
   			  cu = wrk[r].L0 - wrk[r].L1;
	      else cd = wrk[r].L1 - wrk[r].L0;
	      if (wrk[r].L1 >= wrk[r].L2)
   			  cu = cu + wrk[r].L1 - wrk[r].L2;
	      else cd = cd + wrk[r].L2 - wrk[r].L1;
	      if (wrk[r].L2 >= wrk[r].L3)
			     cu = cu + wrk[r].L2 - wrk[r].L3;
	      else cd = cd + wrk[r].L3 - wrk[r].L2;
      }
      val[i]   = (cu+cd != 0) ? cu/(cu+cd) : 0;
      trend[i] = (i<rates_total-1) ? (val[i]>0.75) ? 1 : (val[i]<0.15) ? -1 : (arrowsOnEveryBreak && val[i]<0.75 && val[i]>0.15) ? 0 : trend[i+1] : 0;        
      
      //
      //
      //
     
      if (ShowArrows)
      {
         string lookFor = arrowsId+":"+(string)time[i]; if (ObjectFind(0,lookFor)==0) ObjectDelete(0,lookFor);            
         if (i<(rates_total-1) && trend[i] != trend[i+1])
         {
           if (arrowsOnZoneEnter && trend[i]   ==-1) drawArrow(i,arrowsUpZoneEnterColor,arrowsUpZoneEnterCode,arrowsUpZoneEnterSize,false);
           if (arrowsOnZoneEnter && trend[i]   == 1) drawArrow(i,arrowsDnZoneEnterColor,arrowsDnZoneEnterCode,arrowsDnZoneEnterSize,true);
           if (arrowsOnZoneExit  && trend[i+1] == 1) drawArrow(i,arrowsDnZoneExitColor, arrowsDnZoneExitCode,arrowsDnZoneExitSize,true); 
           if (arrowsOnZoneExit  && trend[i+1] ==-1) drawArrow(i,arrowsUpZoneExitColor, arrowsUpZoneExitCode,arrowsUpZoneExitSize,false);                                                  
         }         
      }
	}
return(rates_total);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------

void drawArrow(int i,color theColor,int theCode,int theSize,bool tup)
{
   string name = arrowsId+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //

      datetime atime = Time[i]; if (arrowsOnNewest) atime += PeriodSeconds(_Period)-1;       
      ObjectCreate(name,OBJ_ARROW,0,atime,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_WIDTH,    theSize);
         ObjectSet(name,OBJPROP_COLOR,    theColor);
         if (tup)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsDisplaceUp * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsDisplaceDn * gap);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int timeFrameValue(int _tf)
{
   int add  = (_tf>=0) ? 0 : fabs(_tf);
   if (add != 0) _tf = _Period;
   int size = ArraySize(iTfTable); 
      int i =0; for (;i<size; i++) if (iTfTable[i]==_tf) break;
                                   if (i==size) return(_Period);
                                                return(iTfTable[(int)fmin(i+add,size-1)]);
}
